package cn.com.qmth.mps.service.impl;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.multipart.MultipartFile;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.boot.tools.excel.model.DataMap;

import cn.com.qmth.mps.bean.User;
import cn.com.qmth.mps.dao.PaperDao;
import cn.com.qmth.mps.entity.CourseEntity;
import cn.com.qmth.mps.entity.ExamEntity;
import cn.com.qmth.mps.entity.PaperEntity;
import cn.com.qmth.mps.enums.Role;
import cn.com.qmth.mps.service.CourseService;
import cn.com.qmth.mps.service.ExamService;
import cn.com.qmth.mps.service.PaperDetailService;
import cn.com.qmth.mps.service.PaperGroupService;
import cn.com.qmth.mps.service.PaperService;
import cn.com.qmth.mps.util.BatchSetDataUtil;
import cn.com.qmth.mps.util.PageUtil;
import cn.com.qmth.mps.vo.exam.ExamPaperCountVo;
import cn.com.qmth.mps.vo.paper.GroupCountVo;
import cn.com.qmth.mps.vo.paper.PaperInfoVo;
import cn.com.qmth.mps.vo.paper.PaperQuery;
import cn.com.qmth.mps.vo.paper.PaperVo;

@Service
public class PaperServiceImpl extends ServiceImpl<PaperDao, PaperEntity> implements PaperService {
	
	@Autowired
	private ExamService examService;
	
	@Autowired
	private CourseService courseService;
	@Autowired
	private PaperGroupService paperGroupService;
	@Autowired
	private PaperDetailService paperDetailService;
	
	@Transactional
	@Override
	public List<String> importPaper(Long examId, User user, MultipartFile file) {
		ExamEntity exam=examService.getById(examId);
		if(exam==null) {
			throw new StatusException("未找到考试批次");
		}
		if (!user.getRole().equals(Role.SUPER_ADMIN) && !user.getSchoolId().equals(exam.getSchoolId())) {
			throw new StatusException("没有权限");
		}
		InputStream inputStream = null;
		try {
			inputStream=file.getInputStream();
			List<DataMap> lineList = ExcelReader.create(ExcelType.XLSX, inputStream, 0).getDataMapList();
			if (CollectionUtils.isEmpty(lineList)) {
				throw new StatusException("Excel无内容");
			}
			if (1001 < lineList.size()) {
				throw new StatusException("数据行数不能超过1000");
			}
			List<String> failRecords = new ArrayList<>();
			List<PaperEntity> ret = new ArrayList<>();
			for (int i = 0; i < lineList.size(); i++) {
				DataMap line = lineList.get(i);

				StringBuilder msg = new StringBuilder();

				PaperEntity imp = new PaperEntity();
				imp.setTotalScore(0.0);
				imp.setObjectiveScore(0.0);
				imp.setSubjectiveScore(0.0);
				imp.setSchoolId(exam.getSchoolId());
				imp.setGroupFinish(false);
				imp.setExamId(examId);
				String code = trimAndNullIfBlank(line.getValue(0));
				if (StringUtils.isBlank(code)) {
					msg.append("  科目代码不能为空");
				} else if (code.length() > 20) {
					msg.append("  科目代码不能超过20个字符");
				}

				String name = trimAndNullIfBlank(line.getValue(1));
				if (StringUtils.isBlank(name)) {
					msg.append("  科目名称不能为空");
				} else if (name.length() > 20) {
					msg.append("  科目名称不能超过20个字符");
				}
				if (msg.length() == 0) {
					CourseEntity course=courseService.saveOrGet(exam.getSchoolId(), code, name);
					imp.setCourseId(course.getId());
				}

				if (msg.length() > 0) {
					failRecords.add(newError(i + 1, msg.toString()));
				} else {
					ret.add(imp);
				}

			}
			if (CollectionUtils.isNotEmpty(failRecords)) {
				return failRecords;
			}
			for (int i = 0; i < ret.size(); i++) {
				PaperEntity cur = ret.get(i);
				try {
					this.save(cur);
				} catch (DuplicateKeyException e) {
					failRecords.add(newError(i + 1, "科目已存在"));
				} catch (Exception e) {
					failRecords.add(newError(i + 1, "系统异常"));
					log.error("科目导入系统异常", e);
				}
			}
			if (CollectionUtils.isNotEmpty(failRecords)) {
				TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
			}
			return failRecords;
		} catch (StatusException e) {
			throw e;
		} catch (Exception e) {
			throw new StatusException("系统错误",e);
		} finally {
			if(inputStream!=null) {
				try {
					inputStream.close();
				} catch (IOException e) {
				}
			}
		}
	}
	
	private String trimAndNullIfBlank(String s) {
		if (StringUtils.isBlank(s)) {
			return null;
		}
		return s.trim();
	}
	private String newError(int lineNum, String msg) {
		return "第" + lineNum + "行" + msg;
	}

	@Override
	public List<ExamPaperCountVo> findPaperCount(List<Long> examIds) {
		return this.baseMapper.findPaperCount(examIds);
	}

	@Override
	public Integer findPaperCount(Long examId) {
		QueryWrapper<PaperEntity> wrapper = new QueryWrapper<>();
		LambdaQueryWrapper<PaperEntity> lw = wrapper.lambda();
		lw.eq(PaperEntity::getExamId, examId);
		return this.count(wrapper);
	}

	@Override
	public PageResult<PaperVo> page(PaperQuery query, User user) {
		if (query.getSchoolId() == null) {
			throw new StatusException("学校不能为空");
		}
		if (!user.getRole().equals(Role.SUPER_ADMIN) && !user.getSchoolId().equals(query.getSchoolId())) {
			throw new StatusException("没有权限");
		}
		IPage<PaperVo> iPage = this.baseMapper.page(new Page<PaperVo>(query.getPageNumber(), query.getPageSize()), query);
		if(CollectionUtils.isNotEmpty(iPage.getRecords())) {
			new BatchSetDataUtil<PaperVo>() {

                @Override
                protected void setData(List<PaperVo> dataList) {
                    List<Long> paperIds = dataList.stream().map(dto -> dto.getId()).distinct()
                            .collect(Collectors.toList());
                    List<GroupCountVo> ret = paperGroupService.findGroupCount(paperIds);
                    if (ret != null && ret.size() > 0) {
                        Map<Long, Integer> countMap = new HashMap<>();
                        for (GroupCountVo item : ret) {
                            countMap.put(item.getPaperId(), item.getGroupCount());
                        }
                        for (PaperVo vo : dataList) {
                            vo.setGroupCount(countMap.get(vo.getId()));
                        }
                    }
                }
            }.setDataForBatch(iPage.getRecords(), 20);
		}
		return PageUtil.of(iPage);
	}

	@Override
	public List<PaperVo> list(Long examId, User user) {
		ExamEntity exam=examService.getById(examId);
		if(exam==null) {
			throw new StatusException("未找到考试批次");
		}
		if (!user.getRole().equals(Role.SUPER_ADMIN) && !user.getSchoolId().equals(exam.getSchoolId())) {
			throw new StatusException("没有权限");
		}
		return this.baseMapper.list(examId);
	}

	@Override
	public PaperInfoVo info(Long id, User user) {
		PaperEntity paper=this.getById(id);
		if(paper==null) {
			throw new StatusException("未找到试卷结构信息");
		}
		if (!user.getRole().equals(Role.SUPER_ADMIN) && !user.getSchoolId().equals(paper.getSchoolId())) {
			throw new StatusException("没有权限");
		}
		PaperInfoVo  vo=new PaperInfoVo();
		BeanUtils.copyProperties(paper, vo);
		CourseEntity course=courseService.getById(vo.getCourseId());
		vo.setCourseCode(course.getCode());
		vo.setCourseName(course.getName());
		vo.setStructInfo(paperDetailService.getStructInfo(vo.getId()));
		vo.setGroupInfo(paperGroupService.getGroupInfo(vo.getId()));
		return vo;
	}

	@Transactional
	@Override
	public List<String> importSubjectStruct(Long examId, User user, MultipartFile file) {
		ExamEntity exam=examService.getById(examId);
		if(exam==null) {
			throw new StatusException("未找到考试批次");
		}
		if (!user.getRole().equals(Role.SUPER_ADMIN) && !user.getSchoolId().equals(exam.getSchoolId())) {
			throw new StatusException("没有权限");
		}
		InputStream inputStream = null;
		try {
			inputStream=file.getInputStream();
			List<DataMap> lineList = ExcelReader.create(ExcelType.XLSX, inputStream, 0).getDataMapList();
			if (CollectionUtils.isEmpty(lineList)) {
				throw new StatusException("Excel无内容");
			}
			if (1001 < lineList.size()) {
				throw new StatusException("数据行数不能超过1000");
			}
			List<String> failRecords = new ArrayList<>();
			List<PaperEntity> ret = new ArrayList<>();
			for (int i = 0; i < lineList.size(); i++) {
				DataMap line = lineList.get(i);

				StringBuilder msg = new StringBuilder();

				PaperEntity imp = new PaperEntity();
				imp.setTotalScore(0.0);
				imp.setObjectiveScore(0.0);
				imp.setSubjectiveScore(0.0);
				imp.setSchoolId(exam.getSchoolId());
				imp.setGroupFinish(false);
				imp.setExamId(examId);
				String code = trimAndNullIfBlank(line.getValue(0));
				if (StringUtils.isBlank(code)) {
					msg.append("  科目代码不能为空");
				} else if (code.length() > 20) {
					msg.append("  科目代码不能超过20个字符");
				}

				String name = trimAndNullIfBlank(line.getValue(1));
				if (StringUtils.isBlank(name)) {
					msg.append("  科目名称不能为空");
				} else if (name.length() > 20) {
					msg.append("  科目名称不能超过20个字符");
				}
				if (msg.length() == 0) {
					CourseEntity course=courseService.saveOrGet(exam.getSchoolId(), code, name);
					imp.setCourseId(course.getId());
				}

				if (msg.length() > 0) {
					failRecords.add(newError(i + 1, msg.toString()));
				} else {
					ret.add(imp);
				}

			}
			if (CollectionUtils.isNotEmpty(failRecords)) {
				return failRecords;
			}
			for (int i = 0; i < ret.size(); i++) {
				PaperEntity cur = ret.get(i);
				try {
					this.save(cur);
				} catch (DuplicateKeyException e) {
					failRecords.add(newError(i + 1, "科目已存在"));
				} catch (Exception e) {
					failRecords.add(newError(i + 1, "系统异常"));
					log.error("科目导入系统异常", e);
				}
			}
			if (CollectionUtils.isNotEmpty(failRecords)) {
				TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
			}
			return failRecords;
		} catch (StatusException e) {
			throw e;
		} catch (Exception e) {
			throw new StatusException("系统错误",e);
		} finally {
			if(inputStream!=null) {
				try {
					inputStream.close();
				} catch (IOException e) {
				}
			}
		}
	}



}
